import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import * as bcrypt from 'bcrypt';
import {
  AuthenticationProps,
  CreateAuthenticationProps,
} from './authentication.types';
import { AuthenticationCreatedDomainEvent } from './events/authentication-created.domain-event';
import { AuthenticationDeletedDomainEvent } from './events/authentication-deleted.domain-event';

export class AuthenticationEntity extends AggregateRoot<AuthenticationProps> {
  protected readonly _id: AggregateID;

  static create = async (
    create: CreateAuthenticationProps,
  ): Promise<AuthenticationEntity> => {
    const props: AuthenticationProps = { ...create };
    const hash = await bcrypt.hash(props.password, 10);
    const authentication = new AuthenticationEntity({
      id: props.userId,
      props: {
        userId: props.userId,
        password: hash,
        usernames: props.usernames,
      },
    });
    authentication.addEvent(
      new AuthenticationCreatedDomainEvent({ aggregateId: props.userId }),
    );
    return authentication;
  };

  delete(): void {
    this.addEvent(
      new AuthenticationDeletedDomainEvent({
        aggregateId: this.id,
      }),
    );
  }

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
