import { AggregateID, AggregateRoot } from '@mobicoop/ddd-library';
import { CreateUsernameProps, UsernameProps } from './username.types';
import { UsernameAddedDomainEvent } from './events/username-added.domain-event';

export class UsernameEntity extends AggregateRoot<UsernameProps> {
  protected readonly _id: AggregateID;

  static create = async (
    create: CreateUsernameProps,
  ): Promise<UsernameEntity> => {
    const props: UsernameProps = { ...create };
    const username = new UsernameEntity({
      id: props.name,
      props: {
        name: props.name,
        userId: props.userId,
        type: props.type,
      },
    });
    username.addEvent(
      new UsernameAddedDomainEvent({ aggregateId: props.name }),
    );
    return username;
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
